(* week-05_strings.ml *)
(* Introduction to Computer Science (YSC1212), Sem2, 2020-2021 *)
(* Olivier Danvy <danvy@yale-nus.edu.sg> *)
(* Version of Fri 19 Feb 2021 *)
(* Version of Thu 11 Mar 2021, with string_mapi *)

(* ********** *)

(* uncomment the following for versions of OCaml that are anterior to 4.02.0: *)

(* #use "anterior-strings.ml" *)

(* ********** *)

let random_char () =
  char_of_int (Random.int 32 + 32);;

let random_string n =
  String.map (fun _ -> random_char ()) (String.make (Random.int n) ' ');;

(* ********** *)

let test_to_a candidate =
  (candidate "" = "") &&
  (candidate "x" = "a") &&
  (candidate "xy" = "aa") &&
  (candidate "xyz" = "aaa")
  (* etc. *);;

let to_a s =
  String.map (fun c -> 'a') s;;

let () = assert (test_to_a to_a);;

(*
   # test_to_a to_a;;
   - : bool = true
   # 
*)

(* ********** *)

(* Exercise 2 *)

(* ***** *)

let test_upper_letter candidate =
  let a = (candidate 'a' = 'A')
  and z = (candidate 'z' = 'Z')
  in a && z;;

let upper_letter c =
  let a = int_of_char 'a'
  and n = int_of_char c
  and z = int_of_char 'z'
  and d = int_of_char 'a' - int_of_char 'A'
  in let () = assert (a <= n && n <= z)
     in char_of_int (n - d);;

let () = assert (test_upper_letter upper_letter);;

let test_upper_string_of_letters candidate =
  (candidate "abc" = "ABC")
  (* etc. *);;

(*
let upper_string_of_letters s =
  ...

let () = assert (test_upper_string_of_letters upper_string_of_letters);;
*)

(* ***** *)

(*
let test_upper_letter candidate =
  ...

let lower_letter c =
  ...

let () = assert (test_lower_letter lower_letter);;
*)

let test_lower_string_of_letters candidate =
  (candidate "ABC" = "abc")
  (* etc. *);;

(*
let lower_string_of_letters s =
  ...

let () = assert (test_lower_string_of_letters lower_string_of_letters);;
*)

(* ********** *)

(* Exercise 3 *)

(* ***** *)

let test_upper_string candidate =
  let b0 = (candidate "abc123" = "ABC123")
  and b1 = (candidate "--ABC**" = "--ABC**")
  and b2 = (candidate "Abc" = "ABC")
  and b3 = (candidate "aBc" = "ABC")
  and b4 = (candidate "abC" = "ABC")
  (* etc. *)
  in b0 && b1 && b2 && b3 && b4;;

(*
let upper_string s =
  ...

let () = assert (test_upper_string upper_string);;
*)

(* ***** *)

let test_lower_string candidate =
  let b0 = (candidate "ABC123" = "abc123")
  and b1 = (candidate "--abc**" = "--abc**")
  and b2 = (candidate "Abc" = "abc")
  and b3 = (candidate "aBc" = "abc")
  and b4 = (candidate "abC" = "abc")
  (* etc. *)
  in b0 && b1 && b2 && b3 && b4;;

(*
let lower_string s =
  ...

let () = assert (test_lower_string lower_string);;
*)

(* ********** *)

let test_blam candidate =
  (candidate "" = "") &&
  (candidate "a" = "0") &&
  (candidate "ab" = "01") &&
  (candidate "abc" = "012") &&
  (candidate "abcd" = "0123")
  (* etc. *);;

let blam s =
  String.mapi (fun i c -> char_of_int (i + 48)) s;;

let () = assert (test_blam blam);;

let blam s =
  String.mapi (fun i c -> char_of_int (i + int_of_char '0')) s;;

let () = assert (test_blam blam);;

(* ********** *)

let lower_case_alphabet =
  String.mapi (fun i _ ->
                 char_of_int (i + int_of_char 'a'))
               (String.make 26 ' ');;

let upper_case_alphabet =
  String.init 26 (fun i -> char_of_int (i + int_of_char 'A'));;

let random_string' n =
  String.init (Random.int n) (fun _ -> random_char ());;

(* ********** *)

let ascii_table =
  String.init 128 char_of_int;;

let test_string_mapi candidate =
  let b0 = (candidate (fun i c -> char_of_int (int_of_char c + 1)) "abcde" = "bcdef")
  and b1 = (candidate (fun i c -> char_of_int (i + int_of_char '0')) "abcde" = "01234")
  in b0 && b1;;

let string_mapi f s =
  let () = assert (String.length s <= 256) in
  String.map
    (fun c -> let i = int_of_char c
              in f i (String.get s i))
    (String.init (String.length s) char_of_int);;

let () = assert (test_string_mapi string_mapi);;

(* ********** *)

(* end of week-05_strings.ml *)

let end_of_file = "week-05_strings.ml";;

(* Digital signature:

           OCaml version 4.01.0
   
   # #use "anterior-strings.ml";;
   module String :
     sig
       ...
     end
   # #use "week-05_strings.ml";;
   val random_char : unit -> char = <fun>
   val random_string : int -> string = <fun>
   val test_to_a : (string -> string) -> bool = <fun>
   val to_a : string -> string = <fun>
   val test_upper_letter : (char -> char) -> bool = <fun>
   val upper_letter : char -> char = <fun>
   val test_upper_string_of_letters : (string -> string) -> bool = <fun>
   val test_lower_string_of_letters : (string -> string) -> bool = <fun>
   val test_upper_string : (string -> string) -> bool = <fun>
   val test_lower_string : (string -> string) -> bool = <fun>
   val test_blam : (string -> string) -> bool = <fun>
   val blam : string -> string = <fun>
   val blam : string -> string = <fun>
   val lower_case_alphabet : string = "abcdefghijklmnopqrstuvwxyz"
   val upper_case_alphabet : string = "ABCDEFGHIJKLMNOPQRSTUVWXYZ"
   val random_string' : int -> string = <fun>
   val ascii_table : string =
     "\000\001\002\003\004\005\006\007\b\t\n\011\012\r\014\015\016\017\018\019\020\021\022\023\024\025\026\027\028\029\030\031 !\"#$%&'()*+,-./0123456789:;<=>?@ABCDEFGHIJKLMNOPQRSTUVWXYZ[\\]^_`abcdefghijklmnopqrstuvwxyz{|}~\127"
   val test_string_mapi : ((int -> char -> char) -> string -> string) -> bool =
     <fun>
   val string_mapi : (int -> char -> char) -> string -> string = <fun>
   val end_of_file : string = "week-05_strings.ml"
   # 
*)
